unit FavOpenDialog;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, Dialogs,buttons,extctrls,stdctrls,commdlg,registry;


type
  TFavOpenDialog = class(TOpenDialog)
  private
    { Private-Deklarationen }
    fpanel : tpanel;
    fbutton : tspeedbutton;
    fcombo     : tcombobox;
    flabel     : tlabel;
    flist      : timagelist;

    ficonheight : integer;
    ficonwidth : integer;
    fovl,fnofold : integer;

    fmaxcount : integer;

    fshowfullpath : boolean;
    fsetinitdir : boolean;

    fstrings : tstrings;
    fassociated : tfavopendialog;
    procedure fbuttonclick(sender:tobject);
    procedure fcomboclick(sender:tobject);
    procedure fcombodraw(Control: TWinControl; Index: Integer ;Rect: TRect;
              State: TOwnerDrawState);
    procedure addfavorite(sr:tstrings;val:string;insert:boolean);
    procedure notification(AComponent: TComponent; Operation: TOperation); override;
    function getfolderpath:string;
  protected
    { Protected-Deklarationen }
    procedure DoClose; override;
    procedure DoShow; override;
    procedure DoFolderChange; override;
    procedure setshowfullpath(val:boolean);
    procedure setstring(index:integer;val:string);
    function getstring(index:integer):string;
    procedure SetFavorites(index:integer;value : tstrings);
    procedure setfassociated(value : tfavopendialog);
  public

    { Public-Deklarationen }
    constructor create(aowner:tcomponent); override;
    destructor Destroy; override;
    function Execute: Boolean; override;
    procedure FavToRegistry(Rootkey:HKEY;Key:string);
    procedure FavFromRegistry (Rootkey:HKEY;Key:string);
  published
    { Published-Deklarationen }
    property MaxCount : integer read fmaxcount write fmaxcount;
    property ShowFullPath : boolean read fshowfullpath write setshowfullpath;
    property ComboHint : string index 1 read getstring
             write setstring ;
    property ButtonHint : string index 2 read getstring
             write setstring ;
    property LabelCaption : string index 3 read getstring
             write setstring ;
    property Favorites : tstrings index 1 read fstrings write setfavorites;
    property Associated : tfavopendialog read fassociated write setfassociated;
    property SetInitDirFrom1st : boolean read fsetinitdir write fsetinitdir;

  end;

  TFavSaveDialog = class(TfavOpenDialog)
    function Execute: Boolean; override;
  end;

procedure Register;

implementation

uses filectrl,shellapi,commctrl;

{$r myfdlg.res}

procedure Register;
begin
  RegisterComponents('Merkes'' Dialogs', [TFavOpenDialog,TFavSaveDialog]);
end;

constructor TFavOpenDialog.create(aowner:tcomponent);
begin
  inherited create(aowner);
  FPanel := TPanel.Create(Self);
  ficonheight := getsystemmetrics(sm_cysmicon);
  ficonwidth := getsystemmetrics(sm_cxsmicon);
  fmaxcount := 10;
  fshowfullpath := false;
  fstrings := tstringlist.create;
  fassociated := nil;
  with FPanel do
  begin
    Name := 'clientPanel';
    Caption := '';
    BevelOuter := bvlowered;
    BorderWidth := 0;
    TabOrder := 1;
    Fcombo := Tcombobox.Create(Self);
    with Fcombo do
    begin
      Name := 'favcombo';
      Parent := FPanel;
      Hint := '';
      parentshowhint := false;
      showhint := true;
      style := csownerdrawfixed;
      ondrawitem := fcombodraw;
      itemheight := ficonheight+2;
      OnClick := fcomboClick;
    end;
    Fbutton := TSpeedButton.Create(Self);
    with FButton do
    begin
      Name := 'addfavButton';
      Enabled := true;
      numglyphs := 2;
      Glyph.LoadFromResourceName(HInstance, 'FAVFOLD');
      Hint := '';
      ParentShowHint := False;
      ShowHint := True;
      OnClick := fbuttonClick;
      Parent := FPanel;
    end;
    Flabel := Tlabel.Create(Self);
    with Flabel do
    begin
      Name := 'favorites';
      caption := 'Favoriten:';
      transparent := true;
      focuscontrol := fcombo;
      ParentShowHint := False;
      ShowHint := false;
      Parent := FPanel;
    end;
    flist := timagelist.create(self);
    with flist do
    begin
      name := 'favimglist';
      bkcolor := clnone;
      shareimages := false;
      fovl:=imagelist_addmasked(handle,loadbitmap(hinstance,'FAVOVL'),cllime);
      fnofold:=imagelist_addmasked(handle,loadbitmap(hinstance,'FAVNOFOLD'),cllime);
    end;
  end;
end;

destructor tfavopendialog.Destroy;
begin
  FButton.Free;
  Fcombo.Free;
  flabel.free;
  flist.free;
  FPanel.Free;
  fstrings.free;
  inherited Destroy;
end;

function tfavopendialog.Execute: Boolean;
begin
  if NewStyleControls and not (ofOldStyleDialog in Options) then
    Template := 'MYDLGO' else
    Template := nil;
  if fsetinitdir then if fstrings.count > 0 then initialdir := fstrings[0];
  Result := inherited execute;
end;

procedure TfavopenDialog.DoClose;
begin
  inherited DoClose;
  { Hide any hint windows left behind }
  Application.HideHint;
  fstrings.assign(fcombo.items);
  if assigned(fassociated) then fassociated.setfavorites(0,fstrings);
end;

procedure TfavopenDialog.DoShow;
var
  PreviewRect: TRect;
  fh : integer;
begin
  { Set preview area to entire dialog }
  GetClientRect(Handle, PreviewRect);
  { Move preview area to right of static area }
  PreviewRect.top := GetStaticRect.bottom;
  FPanel.BoundsRect := PreviewRect;
  FPanel.ParentWindow := Handle;
  fh := (fpanel.clientHeight-25) div 2;
  FButton.setbounds(333,fh,75,25);
  fh := (fpanel.clientHeight-fcombo.height) div 2;
  fcombo.setbounds(80,fh,235,fcombo.height);
  fh := (fpanel.clientHeight-flabel.height) div 2;
  Flabel.setbounds(8,fh,flabel.width,flabel.height);
  inherited DoShow;
  if assigned(fassociated) then setfavorites(0,fassociated.favorites);
  fcombo.items.assign(fstrings);
  fcombo.itemindex := 0;
end;

procedure tfavopendialog.setfavorites(index:integer;value:tstrings);
var ct : integer;
    max : integer;
begin
     fstrings.clear;
     max := value.count -1;
     if max > fmaxcount then max := fmaxcount -1;
     if value.count > 0 then for ct := 0 to max do
        addfavorite(fstrings,value[ct],false);
     if index = 1 then if assigned(fassociated) then fassociated.setfavorites(0,fstrings);
end;

procedure tfavopendialog.addfavorite(sr:tstrings;val:string;insert:boolean);
var ct : integer;
begin
     if val = '' then exit;
     if val[length(val)] = '\' then
        val := copy(val,1,length(val)-1);
     // berprfen, ob schon vorhanden
     if sr.count > 0 then
     for ct := 0 to sr.count-1 do begin
        if lstrcmpi(pchar(sr[ct]),pchar(val)) = 0 then begin
           sr.delete(ct);
           break;
        end;
     end;
     while sr.count >= fmaxcount do sr.delete(sr.count-1);
     if not insert then sr.add(val) else sr.insert(0,val);
end;

function tfavopendialog.getfolderpath:string;
var
    buf : array [0..max_path+1] of char;
begin
     sendmessage(getparent(handle),cdm_getfolderpath,max_path,integer(@buf));
     result := strpas(buf);
     if result <> '' then if result[length(result)] = '\' then delete(result,length(result),1);
end;


procedure tfavopendialog.fbuttonclick(sender:tobject);
var sr :string;
begin
     sr := getfolderpath;
     if fileexists(sr) or directoryexists(sr) or directoryexists(sr+'\') then begin
        addfavorite(fcombo.items,sr,true);
        fcombo.ItemIndex := 0;
        fbutton.enabled := false;
     end;
end;
procedure tfavopendialog.fcomboclick(sender:tobject);
var sr : string;
    sr1 : array [0..max_path+1] of char;
begin
     sr := fcombo.items[fcombo.itemindex];
     if not directoryexists(sr) then exit;
     setfocus(getdlgitem(getparent(handle),1152));
     sendmessage(getdlgitem(getparent(handle),1152),wm_gettext,max_path,integer(@sr1));
     sendmessage(getdlgitem(getparent(handle),1152),wm_settext,0,integer(pchar(sr+'\')));
     sendmessage(getdlgitem(getparent(handle),1),bm_click,0,0);
     sendmessage(getdlgitem(getparent(handle),1152),wm_settext,0,integer(@sr1));
     fcombo.items.delete(fcombo.itemindex);
     fcombo.items.insert(0,sr);
     fcombo.itemindex := 0;
end;
procedure tfavopendialog.fcombodraw(Control: TWinControl; Index: Integer ;Rect: TRect;
              State: TOwnerDrawState);
var sr : string;
    sfi : tshfileinfo;
    lv  : integer;
    tm : ttextmetric;
begin
     if control is tcombobox then with tcombobox(control) do begin
          canvas.fillrect(rect);
          // shell-icon holen
          sr := items[index]+'\';
          lv:=shgetfileinfo(pchar(sr),0,sfi,sizeof(tshfileinfo),
            shgfi_smallicon or shgfi_sysiconindex or shgfi_displayname);
          if (not fshowfullpath) or (length(sr) < 4) then if sfi.szdisplayname <> '' then
             sr := sfi.szdisplayname;
          if (lv <>0) and (sfi.iicon <>0) then begin
             imagelist_draw(lv,sfi.iicon,canvas.handle,rect.left+3,rect.top+2,ild_transparent);
             imagelist_draw(flist.handle,fovl,canvas.handle,rect.left+3,rect.top+2,ild_transparent);
          end else
             imagelist_draw(flist.handle,fnofold,canvas.handle,rect.left+3,rect.top+2,ild_transparent);
          gettextmetrics(canvas.handle,tm);
          textout(canvas.handle,rect.left+8+ficonwidth,(rect.bottom+rect.top-tm.tmheight) shr 1,pchar(sr),length(sr));
     end;
end;

procedure tfavopendialog.setshowfullpath(val:boolean);
begin
     if val <> fshowfullpath then begin
        fshowfullpath := val;
        fcombo.update;
     end;
end;
procedure tfavopendialog.setstring(index:integer;val:string);
begin
     case index of
          1 : fcombo.hint := val;
          2 : fbutton.hint := val;
          3 : flabel.Caption := val;
     end;
end;

function tfavopendialog.getstring(index:integer):string;
begin
     case index of
          1 : result := fcombo.hint;
          2 : result := fbutton.hint;
          3 : result := flabel.caption;
     end;
end;
procedure tfavopendialog.FavToRegistry(Rootkey:HKEY;Key:string);
var reg : tregistry;
    ct : integer;
begin
     reg := tregistry.create;
     try
        reg.RootKey:=rootkey;
        if reg.openkey(key,true) then begin
           ct := fstrings.count;
           reg.writeinteger('Count',ct);
           if ct > 0 then for ct := 0 to ct-1 do
              reg.writestring(inttostr(ct),fstrings[ct]);
        end;
     finally
            reg.free;
     end;
end;

procedure tfavopendialog.FavFromRegistry (Rootkey:HKEY;Key:string);
var reg : tregistry;
    ct : integer;
begin
     reg := tregistry.create;
     try
        reg.RootKey:=rootkey;
        if reg.openkey(key,false) then if reg.valueexists('Count') then begin
           fstrings.clear;
           ct := reg.readinteger('Count');
           if ct > 0 then for ct := 0 to ct-1 do
              fstrings.add(reg.readstring(inttostr(ct)));
        end;
        if assigned(fassociated) then fassociated.setfavorites(0,fstrings);
     finally
            reg.free;
     end;
end;

procedure tfavopendialog.setfassociated(value : tfavopendialog);
begin
     if value <> self then if value <> fassociated then fassociated := value;
end;

procedure tfavopendialog.notification(AComponent: TComponent; Operation: TOperation);
begin
     if operation = opremove then if acomponent = fassociated then fassociated := nil;
     inherited;
end;

procedure tfavopendialog.DoFolderChange;
var sr : string;
    ct : integer;
    ena : boolean;
begin
     // berprfen, ob aktueller folder in history schon drin ist
     sr := getfolderpath;
     ena := true;
     if fcombo.items.count > 0 then for ct := 0 to pred(fcombo.items.count) do begin
        if lstrcmpi(pchar(sr),pchar(fcombo.items[ct])) = 0 then begin
           ena := false;
           fcombo.itemindex := ct;
           break;
        end;
     end;
     fbutton.enabled := ena;
     inherited dofolderchange;
end;


function tfavsavedialog.Execute: Boolean;
begin
  if NewStyleControls and not (ofOldStyleDialog in Options) then
    Template := 'MYDLGO' else
    Template := nil;
  Result := DoExecute(@GetSaveFileName);
end;


end.
